/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.lucis.core.impl;

import static com.google.common.base.Preconditions.checkNotNull;

import java.util.concurrent.Callable;
import java.util.logging.Logger;

import net.sf.lucis.core.Adder;
import net.sf.lucis.core.DirectoryProvider;
import net.sf.lucis.core.IndexException;
import net.sf.lucis.core.IndexStatus;
import net.sf.lucis.core.WriterConfiguration;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriter.MaxFieldLength;

import com.google.common.base.Preconditions;

/**
 * Default adder implementation. Not thread safe.
 * @author Andres Rodriguez.
 */
public final class DefaultAdder implements Adder {
	private final WriterConfiguration config;
	private final DirectoryProvider provider;
	private IndexWriter writer = null;
	private IndexStatus status = IndexStatus.OK;

	/** Log. */
	private Logger log = Logger.getLogger(getClass().getName());

	public DefaultAdder(final WriterConfiguration config, DirectoryProvider provider) {
		this.config = checkNotNull(config, "A writer configuration must be provided");
		this.provider = checkNotNull(provider, "A directory provider must be provided");
	}

	public IndexStatus getStatus() {
		return status;
	}

	/* CONFIGURABLE PROPERTIES */

	public void setLogName(final String name) {
		this.log = Logger.getLogger(name);
	}

	/* END CONFIGURABLE PROPERTIES. */

	private void on() {
		Preconditions.checkState(writer != null, "Adder not started");
	}

	public void add(Document document) {
		add(document, config.getAnalyzer());
	}

	public void add(final Document document, final Analyzer analyzer) {
		on();
		try {
			final Callable<Object> callable = new Callable<Object>() {
				public Object call() throws Exception {
					writer.addDocument(document, analyzer);
					return null;
				}
			};
			MayFail.run(callable);
		} catch (IndexException e) {
			status = e.getStatus();
			throw e;
		}
	}

	void start() {
		if (writer != null) {
			return;
		}
		final Analyzer analyzer = config.getAnalyzer();
		final MaxFieldLength length = config.getMaxFieldLength();
		final Callable<Object> callable = new Callable<Object>() {
			public Object call() throws Exception {
				writer = new IndexWriter(provider.getDirectory(), analyzer, true, length);
				return null;
			}
		};
		status = IndexStatus.OK;
		try {
			MayFail.run(callable);
		} catch (IndexException e) {
			status = e.getStatus();
			throw e;
		}
	}

	void stop() {
		final Callable<Object> callable = new Callable<Object>() {
			public Object call() throws Exception {
				if (status == IndexStatus.OK) {
					writer.commit();
					writer.optimize();
				} else {
					writer.rollback();
				}
				writer.close();
				return null;
			}
		};

	}
}
